#!/bin/bash

remove_old() {
  # Check to see if old printer setup exists, this is for transition to B610s for all the floors (2023-10-20)
  local floor=$1
  
  case $floor in
    11) CHECK="$(lpstat -a | grep "phaser-1115A" 2> /dev/null)"
        if [ -n "$CHECK" ]; then
            echo "Removing old printer setup for $floor floor" 
            sudo lpadmin -x "phaser-1115A-BW-pub" 2> /dev/null
        fi ;;

    12) CHECK="$(lpstat -a | grep "1235A" 2> /dev/null)"
        if [ -n "$CHECK" ]; then
            echo "Removing old printer setup for $floor floor" 
            sudo lpadmin -x "phaser-1235AA-BW-pub" 2> /dev/null
        fi ;;
    13) CHECK="$(lpstat -a | grep "1335A-Versalink" 2> /dev/null)"
        if [ -n "$CHECK" ]; then
            echo "Removing old printer setup for $floor floor" 
            sudo lpadmin -x "1335A-VersalinkB400" 2> /dev/null
            sudo lpadmin -x "1335N-VersalinkB400" 2> /dev/null
        fi ;;

    15) CHECK="$(lpstat -a | grep "XeroxB610-1537" 2> /dev/null)"
        if [ -n "$CHECK" ]; then
            echo "Removing old printer setup for $floor floor" 
            sudo lpadmin -x "XeroxB610-1537-BW" 2> /dev/null
        fi ;;
esac

}

install_printer() {
  local floor=$1
  local printername=$2
  echo "Installing $floor floor printer..."
  # Add ppd file:
  if [ -f /etc/cups/ppd/${printername}.ppd ]; then
    echo "/etc/cups/ppd/${printername}.ppd already exists - would you like to overwrite? (y/n)"
    VALID_ANS=false
    while  [ $VALID_ANS = false ]; do
      read ans
      if [ $ans == "y" ]; then
        sudo cp ${printername}.ppd /etc/cups/ppd/
        sudo chmod go+r /etc/cups/ppd/${printername}.ppd
        VALID_ANS=true
      elif [ $ans == "n" ]; then
        echo ""
        VALID_ANS=true
      else
        echo "Must answer 'y' or 'n':"
      fi
    done
  else
    sudo cp ${printername}.ppd /etc/cups/ppd/
    sudo chmod go+r /etc/cups/ppd/${printername}.ppd
  fi

  # Install printer: (suppress future warning about cups deprecation)
  sudo lpadmin -E -p "${printername}" -v "https://print-spool.math.umass.edu:631/printers/${printername}" -P "/etc/cups/ppd/${printername}.ppd" -o printer-is-shared=false -o auth-info-required=username,password 2> /dev/null

  # Check error status:
  if [ $? != 0 ]; then
    # Notify user and exit script
    echo "Error installing $floor floor printer"
    exit 255
  else
    sudo cupsenable $printername
    if [ $? != 0 ]; then
      echo "Error installing $floor floor printer"
      exit 255
    else
      sudo cupsaccept $printername
      if [ $? != 0 ]; then
        echo "Error installing $floor floor printer"
        exit 255
      else
        echo "Installed $floor floor printer"
        echo ""
      fi
    fi
  fi
}


check_and_install() {
  local floor=$1
  local printername=$2

  # Check if printer already installed: (while suppressing stderr)
  CHECK="$(lpstat -a ${printername} 2> /dev/null)"
  # If not installed, do so:
  if [ -z "$CHECK" ]; then    
    remove_old $floor
    install_printer $floor $printername
  else
    echo "$floor floor printer is already installed - would you like to reinstall? (y/n)"
    VALID_ANS=false
    while  [ $VALID_ANS = false ]; do
      read ans
      if [ $ans == "y" ]; then
        remove_old $floor
	install_printer $floor $printername
        # Break out of while loop
        VALID_ANS=true
      elif [ $ans == "n" ]; then
        VALID_ANS=true
        echo ""
        # Exit out of function
        return 0
      else
        echo "Must answer 'y' or 'n':"
      fi
    done
  fi
}

###############################################################################
###############################################################################
###############################################################################

sudo echo ""
if [ $? != 0 ]; then
  echo "Permissions Error... exiting"
  exit 255
fi

# Make a backup of current cups configuration:
if [ ! -d /etc/cups-$(date +%Y%m%d) ]; then
  echo "Backing up current printing configuration to /etc/cups-$(date +%Y%m%d)/"
  sudo mkdir /etc/cups-$(date +%Y%m%d)/
  sudo cp -R /etc/cups/* /etc/cups-$(date +%Y%m%d)/
  if [ $? != 0 ]; then
    echo "Backup Error... exiting"
  fi
  echo ""
fi

# Install any missing printers:
check_and_install "16" "1637-XeroxB610DN"
check_and_install "15" "1585-XeroxB610DN"
check_and_install "14" "1435A-XeroxB610DN"
check_and_install "13" "1335N-VersalinkB400" # Change name after changing printer to new model
check_and_install "12" "1235A-XeroxB610DN"
check_and_install "11" "1115A-XeroxB610DN"

# Show available printers
echo "Available Printers:"
lpstat -a
